//-----------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------
var ACCESS_PROTOCOL = "https";
var ACCESS_PORT = "443";
var ACCESS_NAME_CLIENT = "niac";
var ACCESS_NAME_SERVER = "nias";
//-----------------------------------------------------------------------
// Global
//-----------------------------------------------------------------------
var CB_ThisObject = null;
//-----------------------------------------------------------------------
// ConnectInfo
//-----------------------------------------------------------------------
function ConnectInfo(name, version, image, keyid, functions, site) {
    this.name = name;
    this.version = version;
    this.image = image;
    this.keyid = keyid;
    this.functions = functions;
    this.site = site;
}
//-----------------------------------------------------------------------
// UserInfo
//-----------------------------------------------------------------------
function UserInfo(personalNumber, name, company, token, certID, certValue, sharedSecret) {
    this.personalNumber = personalNumber;
    this.name = name;
    this.company = company;
    this.token = token;
    this.certificateID = certID;
    this.certificateValue = certValue;
    this.sharedSecret = null;
    if ((sharedSecret != null) && (sharedSecret.length > 0)) {
        this.sharedSecret = sharedSecret;
    }
}
//-----------------------------------------------------------------------
// RequestInfo
//-----------------------------------------------------------------------
function RequestInfo(address, personalNumber, orderRef, type, serverName, serverImage, visibleData, nonVisibleData, date) {
    this.address = address;
    this.personalNumber = personalNumber;
    this.orderRef = orderRef;
    this.type = type;
    this.serverName = serverName;
    this.serverImage = serverImage;
    this.visibleData = visibleData;
    this.nonVisibleData = nonVisibleData;
    this.date = date;
    this.status = "";
}
//-----------------------------------------------------------------------
// CollectInfo
//-----------------------------------------------------------------------
function CollectInfo(progressStatus, signature, userInfo) {
    this.progressStatus = progressStatus;
    this.signature = signature;
    this.userInfo = userInfo;
}
//-----------------------------------------------------------------------
// AccessClient
//-----------------------------------------------------------------------
function AccessClient(address) {
    //-----------------------------------------------------------------------
    // Members
    //-----------------------------------------------------------------------
    this.address = address;
    this.url = ACCESS_PROTOCOL + "://" + this.address + ":" + ACCESS_PORT + "/" + ACCESS_NAME_CLIENT + "/";
    this.callback = null;
    this.context = null;
    this.info = null;
    this.user = null;
    this.request = null;
    this.status = null;
    //-----------------------------------------------------------------------
    // SetCallback
    //-----------------------------------------------------------------------
    this.SetCallback = function(callback, context) {
        this.callback = callback;
        this.context = context;
        return;
    }
    //-----------------------------------------------------------------------
    // GetURL
    //-----------------------------------------------------------------------
    this.GetURL = function() {
        return this.url;
    }
    //-----------------------------------------------------------------------
    // GetSiteURL
    //-----------------------------------------------------------------------
    this.GetSiteURL = function(site) {
        var value = null;
        var i = 0;
        if ((site != null) && (site.length > 0)) {
            if ((i = this.url.lastIndexOf(':')) != -1) {
                if ((i = this.url.indexOf('/', i)) != -1) {
                    value = this.url.substr(0, i + 1) + site + "/";
                }
            }
        }
        return value;
    }
    //-----------------------------------------------------------------------
    // GetConnectInfo
    //-----------------------------------------------------------------------
    this.GetConnectInfo = function() {
        var params = new SOAPClientParameters();
        CB_ThisObject = this;
        try {
            this.info = null;
            SOAPClient.invoke(this.url, "GetInfo", params, (this.callback != null), this.GetConnectInfoCB);
        } catch (ex) {
            this.info = null;
        }
        return this.info;
    }
    //-----------------------------------------------------------------------
    // GetConnectInfoCB
    //-----------------------------------------------------------------------
    this.GetConnectInfoCB = function(connectInfo) {
        CB_ThisObject.info = null;
        if (connectInfo != null) {
            CB_ThisObject.info = new ConnectInfo(
                connectInfo.name,
                connectInfo.version,
                connectInfo.image,
                connectInfo.keyid,
                connectInfo.functions,
                connectInfo.site);
        }
        if (CB_ThisObject.callback != null) {
            CB_ThisObject.callback(info, CB_ThisObject.context);
        }
        return;
    }
    //-----------------------------------------------------------------------
    // AddUser
    //-----------------------------------------------------------------------
    this.AddUser = function(certificate) {
        var params = new SOAPClientParameters();
        params.add("certificate", certificate);
        CB_ThisObject = this;
        try {
            this.user = null;
            SOAPClient.invoke(this.url, "UserAdd", params, (this.callback != null), this.AddUserCB);
        } catch (ex) {
            this.user = null;
        }
        return this.user;
    }
    //-----------------------------------------------------------------------
    // AddUserCB
    //-----------------------------------------------------------------------
    this.AddUserCB = function(userInfo) {
        var user = null;
        if (userInfo != null) {
            user = new UserInfo(
                userInfo.personalNumber,
                userInfo.name,
                userInfo.company,
                "",
                "",
                "",
                userInfo.sharedSecret);
        }
        CB_ThisObject.user = user;
        if (CB_ThisObject.callback != null) {
            CB_ThisObject.callback(user, CB_ThisObject.context);
        }
        return;
    }
    //-----------------------------------------------------------------------
    // GetUser
    //-----------------------------------------------------------------------
    this.GetUser = function(personalNumber) {
        var params = new SOAPClientParameters();
        params.add("personalNumber", personalNumber);
        CB_ThisObject = this;
        try {
            this.user = null;
            SOAPClient.invoke(this.url, "UserInfo", params, (this.callback != null), this.GetUserCB);
        } catch (ex) {
            this.user = null;
        }
        return this.user;
    }
    //-----------------------------------------------------------------------
    // GetUserCB
    //-----------------------------------------------------------------------
    this.GetUserCB = function(userInfo) {
        var user = null;
        if (userInfo != null) {
            user = new UserInfo(
                userInfo.personalNumber,
                userInfo.name,
                "",
                "",
                "",
                "",
                null);
        }
        CB_ThisObject.user = user;
        if (CB_ThisObject.callback != null) {
            CB_ThisObject.callback(user, CB_ThisObject.context);
        }
        return;
    }
    //-----------------------------------------------------------------------
    // RegisterRequest
    //-----------------------------------------------------------------------
    this.RegisterRequest = function(orderRef, personalNumber) {
        var params = new SOAPClientParameters();
        params.add("orderRef", orderRef);
        params.add("personalNumber", personalNumber);
        CB_ThisObject = this;
        this.user = personalNumber;
        try {
            this.request = null;
            SOAPClient.invoke(this.url, "RequestRegister", params, (this.callback != null), this.RegisterRequestCB);
        } catch (ex) {
            this.request = null;
        }
        return this.request;
    }
    //-----------------------------------------------------------------------
    // RegisterRequestCB
    //-----------------------------------------------------------------------
    this.RegisterRequestCB = function(requestInfo) {
        var request = null;
        if ((requestInfo != null) && (requestInfo.orderRef != null)) {
            request = new RequestInfo(
                CB_ThisObject.address,
                CB_ThisObject.user,
                requestInfo.orderRef,
                requestInfo.type,
                requestInfo.serverName,
                requestInfo.serverImage,
                requestInfo.visibleData,
                requestInfo.nonVisibleData,
                requestInfo.date);
        }
        CB_ThisObject.request = request;
        if (CB_ThisObject.callback != null) {
            CB_ThisObject.callback(request, CB_ThisObject.context);
        }
        return;
    }    
    //-----------------------------------------------------------------------
    // NextRequest
    //-----------------------------------------------------------------------
    this.NextRequest = function(personalNumber) {
        var params = new SOAPClientParameters();
        params.add("personalNumber", personalNumber);
        CB_ThisObject = this;
        this.user = personalNumber;
        try {
            this.request = null;
            SOAPClient.invoke(this.url, "RequestNext", params, (this.callback != null), this.NextRequestCB);
        } catch (ex) {
            this.request = null;
        }
        return this.request;
    }
    //-----------------------------------------------------------------------
    // NextRequestCB
    //-----------------------------------------------------------------------
    this.NextRequestCB = function(requestInfo) {
        var request = null;
        if ((requestInfo != null) && (requestInfo.orderRef != null)) {
            request = new RequestInfo(
                CB_ThisObject.address,
                CB_ThisObject.user,
                requestInfo.orderRef,
                requestInfo.type,
                requestInfo.serverName,
                requestInfo.serverImage,
                requestInfo.visibleData,
                requestInfo.nonVisibleData,
                requestInfo.date);
        }
        CB_ThisObject.request = request;
        if (CB_ThisObject.callback != null) {
            CB_ThisObject.callback(request, CB_ThisObject.context);
        }
        return;
    }
    //-----------------------------------------------------------------------
    // ProcessRequest
    //-----------------------------------------------------------------------
    this.ProcessRequest = function(orderRef, action, signature, device) {
        var params = new SOAPClientParameters();
        params.add("orderRef", orderRef);
        params.add("action", action);
        params.add("device", device.name + " " + device.version);
        if (signature != null) {
            params.add("signature", signature);
        }
        CB_ThisObject = this;
        try {
            this.status = null;
            SOAPClient.invoke(this.url, "RequestProcess", params, (this.callback != null), this.ProcessRequestCB);
        } catch (ex) {
            this.status = null;
        }
        return this.status;
    }
    //-----------------------------------------------------------------------
    // ProcessRequestCB
    //-----------------------------------------------------------------------
    this.ProcessRequestCB = function(status) {
        CB_ThisObject.status = status;
        if (CB_ThisObject.callback != null) {
            CB_ThisObject.callback(status, CB_ThisObject.context);
        }
        return;
    }
}
//-----------------------------------------------------------------------
// AccessServer
//-----------------------------------------------------------------------
function AccessServer(address) {
    //-----------------------------------------------------------------------
    // Members
    //-----------------------------------------------------------------------
    this.address = address;
    this.url = ACCESS_PROTOCOL + "://" + this.address + ":" + ACCESS_PORT + "/" + ACCESS_NAME_SERVER + "/";
    this.callback = null;
    this.context = null;
    this.orderRef = null;
    this.collect = null;
    //-----------------------------------------------------------------------
    // SetCallback
    //-----------------------------------------------------------------------
    this.SetCallback = function(callback, context) {
        this.callback = callback;
        this.context = context;
        return;
    }
    //-----------------------------------------------------------------------
    // GetURL
    //-----------------------------------------------------------------------
    this.GetURL = function() {
        return this.url;
    }
    //-----------------------------------------------------------------------
    // Authenticate
    //-----------------------------------------------------------------------
    this.Authenticate = function(personalNumber) {
        var params = new SOAPClientParameters();
        params.add("personalNumber", personalNumber);
        CB_ThisObject = this;
        try {
            this.orderRef = null;
            SOAPClient.invoke(this.url, "Authenticate", params, (this.callback != null), this.AuthenticateCB);
        } catch (ex) {
            this.orderRef = null;
        }
        return this.orderRef;
    }
    //-----------------------------------------------------------------------
    // AuthenticateCB
    //-----------------------------------------------------------------------
    this.AuthenticateCB = function(orderRef) {
        CB_ThisObject.orderRef = orderRef;
        if (CB_ThisObject.callback != null) {
            CB_ThisObject.callback(orderRef, CB_ThisObject.context);
        }
        return;
    }
    //-----------------------------------------------------------------------
    // Sign
    //-----------------------------------------------------------------------
    this.Sign = function(personalNumber, visibleData, nonVisibleData) {
        var params = new SOAPClientParameters();
        params.add("personalNumber", personalNumber);
        params.add("userVisibleData", visibleData);
        params.add("userNonVisibleData", nonVisibleData);
        CB_ThisObject = this;
        try {
            this.orderRef = null;
            SOAPClient.invoke(this.url, "Sign", params, (this.callback != null), this.SignCB);
        } catch (ex) {
            this.orderRef = null;
        }
        return this.orderRef;
    }
    //-----------------------------------------------------------------------
    // SignCB
    //-----------------------------------------------------------------------
    this.SignCB = function(orderRef) {
        CB_ThisObject.orderRef = orderRef;
        if (CB_ThisObject.callback != null) {
            CB_ThisObject.callback(orderRef, CB_ThisObject.context);
        }
        return;
    }
    //-----------------------------------------------------------------------
    // Collect
    //-----------------------------------------------------------------------
    this.Collect = function(orderRef) {
        var params = new SOAPClientParameters();
        params.add("orderRef", orderRef);
        CB_ThisObject = this;
        try {
            this.collect = null;
            SOAPClient.invoke(this.url, "Collect", params, (this.callback != null), this.CollectCB);
        } catch (ex) {
            this.collect = null;
        }
        return this.collect;
    }
    //-----------------------------------------------------------------------
    // CollectCB
    //-----------------------------------------------------------------------
    this.CollectCB = function(collectInfo) {
        var collect = null;
        if (collectInfo != null) {
            collect = new CollectInfo(
                collectInfo.progressStatus,
                collectInfo.signature,
                collectInfo.userInfo);
        }
        CB_ThisObject.collect = collect;
        if (CB_ThisObject.callback != null) {
            CB_ThisObject.callback(collect, CB_ThisObject.context);
        }
        return;
    }
}
