//-----------------------------------------------------------------------
// DataStore
//-----------------------------------------------------------------------
function DataStore() {
    //-----------------------------------------------------------------------
    // getValue
    //-----------------------------------------------------------------------
    this.getValue = function(name) {
        var value = "";
        var i = 0;
        var count = 0;
        var part = null;
        // Use internal if available
        if (this.useInternal()) {
            value = this.iidGetValue(name);
        }
        else {
            // Value is separated into blocks
            while ((part = this._getCookie(this._getName(name, count))) != null) {
                if (part.length == 0)
                    break;
                value += part;
                count++;
            }
            // Empty value used as non-existing
            if (value.length > 0) {
                value = unescape(value);
            }
            else {
                value = null;
            }
        }
        return value;
    }
    //-----------------------------------------------------------------------
    // setValue
    //-----------------------------------------------------------------------
    this.setValue = function(name, value) {
        var date = new Date();
        var blockSize = 0;
        var i = 0;
        var part = null;
        var count = 0;
        var expire = null;
        // Use internal if available
        if (this.useInternal()) {
            this.iidSetValue(name, value);
        }
        else {
            // For null store as empty value and set expire date to yesterday, 
            // and it will be removed by browser
            if (value == null) {
                value = "";
                date.setDate(date.getDate() - 1);
            }
            // Need some expire, use 2 years
            else {
                date.setDate(date.getDate() + 2 * 365);
            }
            if (value.length > 0) {
                value = escape(value);
                // Limiation of size, so separate in 4K blocks
                blockSize = 4 * 1024;
                count = 0;
                while ((count == 0) || (i < value.length)) {
                    if ((value.length - i) > blockSize) {
                        part = value.substr(i, blockSize);
                    }
                    else {
                        part = value.substr(i, value.length - i);
                    }
                    this._setCookie(this._getName(name, count), part, date.toUTCString());
                    i += blockSize;
                    count++;
                }
            }
            // Remove eventual last (set with expire)
            if (this._getCookie(this._getName(name, count)) != null) {
                date.setDate(date.getDate() - 1);
                this._setCookie(this._getName(name, count), "", date);
            }
            // Test value is set correct
            if ((value != null) && (value.length > 0)) {
                part = this.getValue(name);
                part = escape(part);
                if (part != value) {
                    // Asch failure, assume cookies are corrupt so reset all
                    this.resetAll();
                }
            }
        }
        return;
    }
    //-----------------------------------------------------------------------
    // useInternal
    //-----------------------------------------------------------------------
    this.useInternal = function() {
        var should = false;
        if (iid_IsAvailable()) {
            should = true;
        }
        return should;
    }
    //-----------------------------------------------------------------------
    // useLocalStorage
    //-----------------------------------------------------------------------
    this.useLocalStorage = function() {
        var found = false;
        try {
            if (typeof window.localStorage == 'object') {
                if ((window.localStorage.getItem != null) &&
                    (window.localStorage.setItem != null)) {
                    found = true;
                }
            }
        }
        catch (err) {
            found = false;
        }
        return found;
    }
    //-----------------------------------------------------------------------
    // resetAll
    //-----------------------------------------------------------------------
    this.resetAll = function() {
        this._deleteAllCookies();
        if (this.useInternal()) {
            this.iidResetAll();
        }
        return;
    }
    //-----------------------------------------------------------------------
    // _getName
    //-----------------------------------------------------------------------
    this._getName = function(name, count) {
        var value = "iid_";
        if (name != null) {
            value += name + "_" + count;
        }
        return value;
    }
    //-----------------------------------------------------------------------
    // _getCookie
    //-----------------------------------------------------------------------
    this._getCookie = function(name) {
        var value = null;
        var i = 0;
        var str1 = null;
        var str2 = null;
        // Use localStorage variable if available
        if (this.useLocalStorage()) {
            value = window.localStorage.getItem(name);
        }
        else {
            var cookies = document.cookie.split(";");
            for (i = 0; i < cookies.length; i++) {
                str1 = cookies[i].substr(0, cookies[i].indexOf("="));
                str2 = cookies[i].substr(cookies[i].indexOf("=") + 1);
                str1 = str1.replace(/^\s+|\s+$/g, "");
                // Empty value used as non-existing
                if (str1 == name) {
                    if (str2.length > 0) {
                        value = str2;
                    }
                    break;
                }
            }
        }
        return value;
    }
    //-----------------------------------------------------------------------
    // _setCookie
    //-----------------------------------------------------------------------
    this._setCookie = function(name, value, date) {
        // Use localStorage variable if available
        if ((name != null) && (name.length > 0)) {
            if (this.useLocalStorage()) {
                window.localStorage.setItem(name, value);
            }
            else {
                // Cookies available for all pages
                value += "; path=/";
                // Format cookie value
                if (date != null) {
                    value += "; expires=" + date;
                }
                document.cookie = name + "=" + value;
            }
            if (value.length > 20)
                value = value.substr(0, 20);
        }
        return;
    }
    //-----------------------------------------------------------------------
    // _deleteAllCookies
    //-----------------------------------------------------------------------
    this._deleteAllCookies = function() {
        // Use localStorage variable if available
        if (this.useLocalStorage()) {
            window.localStorage.clear();
        }
        else {
            var cookies = document.cookie.split(";");
            for (var i = 0; i < cookies.length; i++) {
                var cookie = cookies[i];
                var pos = cookie.indexOf("=");
                var name = pos > -1 ? cookie.substr(0, pos) : cookie;
                document.cookie = name + "=;expires=Thu, 01 Jan 1970 00:00:00 GMT";
            }
        }
        return;
    }
    //-----------------------------------------------------------------------
    // iidName
    //-----------------------------------------------------------------------
    this.iidName = function(name) {
        return "ConfigLocal:Temp:" + name;
    }
    //-----------------------------------------------------------------------
    // iidGetValue
    //-----------------------------------------------------------------------
    this.iidGetValue = function(name) {
        var value = null;
        if ((value = iid_GetProperty(this.iidName(name))) != "") {
            value = BASE64.decode(value);
        }
        else {
            value = null;
        }
        return value;
    }
    //-----------------------------------------------------------------------
    // iidSetValue
    //-----------------------------------------------------------------------
    this.iidSetValue = function(name, value) {
        if (value == null) {
            value = "";
        }
        else {
            value = BASE64.encode(value);
        }
        iid_SetProperty(this.iidName(name), value);
        return;
    }
    //-----------------------------------------------------------------------
    // iidResetAll
    //-----------------------------------------------------------------------
    this.iidResetAll = function() {
        iid_SetProperty("ConfigLocal:Temp:", "");
        return;
    }
}
