//-----------------------------------------------------------------------
// Globals
//-----------------------------------------------------------------------
var USER = null;
var REQUEST = null;
//-----------------------------------------------------------------------
// OnLoad
//-----------------------------------------------------------------------
function OnLoad() {
    var vars = null;
    if ((vars = GetVariables("activate")) != null) {
        SetCurrentLanguage(vars);
        OnLoadVariables(vars);
    }
    return;
}
//-----------------------------------------------------------------------
// OnLoadVariables
//-----------------------------------------------------------------------
function OnLoadVariables(vars) {
    var store = null;
    var requestInfo = null;
    var request = null;
    var userInfo = null;
    var user = null;
    var certificate = null;
    if (vars != null) {
        store = new DataStore();
        // First call will we get request and user, the second call will get
        // certificate
        if (((userInfo = vars["user"]) != null) &&
            ((requestInfo = vars["request"]) != null)) {
            store.setValue("user", userInfo);
            store.setValue("request", requestInfo);
            // Parse user and request info (should always succeed)
            if (((user = ParseUserInfo(userInfo)) != null) &&
                ((request = ParseRequestInfo(requestInfo)) != null)) {
                // Set user information
                SetInformation(user, request);
                // Load certificate from plugin
                OnLoadCertificate(user);
            }
        }
        else {
            userInfo = store.getValue("user");
            requestInfo = store.getValue("request");
            certificate = vars["Certificate"];
            // Parse user and request info (should always succeed)
            if (((user = ParseUserInfo(userInfo)) != null) &&
                ((request = ParseRequestInfo(requestInfo)) != null)) {
                // Set user information
                SetInformation(user, request);
                // First call we get certificate, the following calls will get 
                // eventual errors or completed when data is available
                if (certificate != null) {
                    OnAddUser(user, request, certificate);
                }
                // View will return without result, so ignore
                else if (vars["result"] != null) {
                    OnDecryptDone(user, request, vars["result"], vars["Data"]);
                }
                else {
                    SetElementVisible("label_password");
                    SetElementVisible("pwd");
                    SetFieldFocus("pwd");
                }
                // Store values
                USER = user;
                REQUEST = request;
            }
        }
    }
    return;
}
//-----------------------------------------------------------------------
// OnLoadCertificate
//-----------------------------------------------------------------------
function OnLoadCertificate(user) {
    var value = null;
    ShowProgress();
    value = "Base64=true;";
    value += "Certificate=" + URL.encode(GetPartBy(GetPartBy(user.certificateID, 0, ','), 0, '|')) + ";";
    CallPluginComponent(document.form1.next.value, "plugin", value, "", "", "Certificate", OnLoadVariables);     
    return;
}
//-----------------------------------------------------------------------
// OnAddUser
//-----------------------------------------------------------------------
function OnAddUser(user, request, certificate) {
    var client = null;
    ShowProgress();
    // Remove eventual url encoding
    if (certificate.indexOf("%") != -1) {
        certificate = URL.decode(certificate);
    }
    // Send certificate to server and get user info back
    client = new AccessClient(request.address);
    client.callback = OnProcessDone;
    client.AddUser(certificate);
    return;
}
//-----------------------------------------------------------------------
// OnProcessDone
//-----------------------------------------------------------------------
function OnProcessDone(user) {
    var store = null;
    // May not be accepted
    if (user == null) {
        HandleResult("CKR_FUNCTION_FAILED");
    }
    // Should never fail, but check anyway
    else if ((user.sharedSecret == null) || (user.sharedSecret.length == 0)) {
        HandleResult("CKR_ENCRYPTED_DATA_INVALID");
    }
    else {
        store = new DataStore();
        store.setValue("encrypted_data", user.sharedSecret);
    }
    SetElementVisible("label_password");
    SetElementVisible("pwd");
    SetFieldFocus("pwd");
    // Too fast, so wait with hide of progress
    setTimeout(OnProcessDoneDelayed, 333);
    return;
}
//-----------------------------------------------------------------------
// OnProcessDoneDelayed
//-----------------------------------------------------------------------
function OnProcessDoneDelayed() {
    HideProgress();
    return;
}
//-----------------------------------------------------------------------
// OnDecryptDone
//-----------------------------------------------------------------------
function OnDecryptDone(user, request, result, data) {
    var value = null;
    var info = null;
    if (result != null) {
        if ((result == "CKR_OK") && (data != null)) {
            if ((info = new ActiveUserInfo(
                request.address,
                user.personalNumber,
                user.name,
                user.company, 
                null)) != null) {
                SetActiveUserInfo(info, data);
            }
            LoadPage("index", null);
        }
        else {
            HandleResult(result);
        }
    }
    return;
}
//-----------------------------------------------------------------------
// SetInformation
//-----------------------------------------------------------------------
function SetInformation(user, request) {
    SetElementValue("title", GetRequestString(request.type));
    SetElementValue("text_btn_ok", GetRequestString(request.type));
    SetElementLabel("companyName", request.serverName);
    SetElementImage("companyImage", GetImageByPath(request.serverImage, request.address));
    SetElementLabel("userName", user.name);
    SetElementImage("userImage", GetBigImageByName(user.company, true));
    SetElementValue("personalNumber", FormatPersonalNumber(user.personalNumber));
    SetElementLabel("tokenName", GetPartBy(user.token, 0, ","));
    SetElementValue("warning", "&nbsp;");
    SetElementLabel("label_password", GetString("TEXT_PIN"));
    ShowHeader();
    SetElementHidden("label_password");
    SetElementHidden("pwd");
    // Will start with hidden view, so we avoid double load when reading
    // certificate
    SetElementVisible("content");
    return;
}
//-----------------------------------------------------------------------
// OnBack
//-----------------------------------------------------------------------
function OnBack() {
    LoadPage("index", null);
    return;
}
//-----------------------------------------------------------------------
// OnButtonView
//-----------------------------------------------------------------------
function OnButtonView() {
    var store = new DataStore();
    var value = "";
    value += "back=activate&";
    value += "title=" + URL.encode(GetRequestString(REQUEST.type)) + "&";
    value += "data=" + URL.encode(REQUEST.nonVisibleData);
    LoadPage("view", value);
    return;
}
//-----------------------------------------------------------------------
// OnButtonOK
//-----------------------------------------------------------------------
function OnButtonOK() {
    SetFieldUnfocus("pwd");
    ShowProgress();
    setTimeout(OnDecrypt, 111);
    return;
}
//-----------------------------------------------------------------------
// OnDecrypt
//-----------------------------------------------------------------------
function OnDecrypt() {
    var store = null;
    store = new DataStore();
    store.getValue("encrypted_data");
    DoActionDecrypt(store.getValue("encrypted_data"));
    return;
}
//-----------------------------------------------------------------------
// HandleResult
//-----------------------------------------------------------------------
function HandleResult(result) {
    HideProgress();
    if (result != "CKR_OK") {
        ShowErrorPkcs11(result);
    }
    else {
        store.setValue("encrypted_data", null);
    }
    return;
}
//-----------------------------------------------------------------------
// DoActionDecrypt
//-----------------------------------------------------------------------
function DoActionDecrypt(data) {
    var value = "";
    // Need a pin value
    if (document.form1.pwd.value.length == 0) {
        HandleResult("CKR_PIN_INVALID");
    }
    else if ((data == null) || (data.length == 0)) {
        HandleResult("CKR_ENCRYPTED_DATA_INVALID");
    }
    else {
        // Set plugin parameters
        value += "Base64=true;";
        value += "UrlEncode=false;";
        value += "Data=" + data + ";";
        value += "Password=" + URL.encode(document.form1.pwd.value) + ";";
        CallPluginComponent(document.form1.next.value, "plugin", value, "Decrypt", "", "Data", OnLoadVariables)
    }
    return;
}        
