#!/bin/bash

if [ $(uname) = "Darwin" ]; then
    MODUTIL=/opt/local/bin/nss-modutil
    LIBEXT=dylib
    FIREFOX_ROOT=/Applications/Firefox.app/Contents/MacOS/
    FIREFOX_PROFILES="$HOME/Library/Application Support/Firefox/Profiles/"
    THUNDERBIRD_PROFILES="$HOME/Library/Thunderbird/Profiles/"
    CHROME_NSSDB=$HOME/.pki/nssdb
else  
    MODUTIL=/usr/bin/modutil
    LIBEXT=so
    FIREFOX_ROOT=/usr/lib/firefox/
    FIREFOX_PROFILES=$HOME/.mozilla/firefox/
    THUNDERBIRD_ROOT=/usr/lib/thunderbird/
    THUNDERBIRD_PROFILES="$HOME/.thunderbird/"
    CHROME_NSSDB=$HOME/.pki/nssdb
    if [ ! -d FIREFOX_ROOT ]; then
        FF_TMP=$(ls /usr/lib | grep firefox)
        FF_NAME_2=${FF_TMP:0:11}
        FF_NAME_3=${FF_TMP:0:13}
        if [ -f "/usr/lib/$FF_NAME_2/libnss3.so" ]; then
            FIREFOX_ROOT=/usr/lib/$FF_NAME_2/
        elif [ -f "/usr/lib/$FF_NAME_3/libnss3.so" ]; then 
            FIREFOX_ROOT=/usr/lib/$FF_NAME_3/
        fi
    fi
    if [ ! -d THUNDERBIRD_ROOT ]; then
        TB_TMP=$(ls /usr/lib | grep thunderbird)
        TB_NAME_2=${TB_TMP:0:11}
        TB_NAME_3=${TB_TMP:0:13}
        if [ -f "/usr/lib/$TB_NAME_2/libnss3.so" ]; then
            THUNDERBIRD_ROOT=/usr/lib/$TB_NAME_2/
        elif [ -f "/usr/lib/$TB_NAME_3/libnss3.so" ]; then 
            THUNDERBIRD_ROOT=/usr/lib/$TB_NAME_3/
        fi
    fi
fi

if [ -f "/usr/lib/libiidp11.$LIBEXT" ]; then
    PKCS11LIB=/usr/lib/libiidp11.$LIBEXT
    PKCS11BIN=/usr/bin/iid
else
    PKCS11LIB=$HOME/.iid/lib/libiidp11.$LIBEXT
    PKCS11BIN=$HOME/.iid/bin/iid
fi

if [ -f "$PKCS11LIB" ]; then 
    if [ -d "$FIREFOX_PROFILES" ] || [ -d "$THUNDERBIRD_PROFILES" ] || [ -d "$CHROME_NSSDB" ]; then
        if [ $(uname) != "Darwin" ]; then
            if [ -f $MODUTIL ]; then
                echo "Tool \"modutil\" already installed"
            else
                sudo apt-get update
                sudo apt-get install libnss3-tools
            fi
        fi
        if [ -f $MODUTIL ]; then
            if [ -d "$FIREFOX_PROFILES" ]; then
                FF_PROFILES=$(ls "$FIREFOX_PROFILES" | grep .default)
                for i in ${FF_PROFILES[@]}; do
                    echo "Found Firefox profile: '$FIREFOX_PROFILES$i'"
                    FF_LIST=$($MODUTIL -dbdir "$FIREFOX_PROFILES$i" -list | grep $PKCS11LIB)
                    if [ $(echo ${#FF_LIST}) -gt 10 ]; then 
                        echo "Module \"Net iD\" pkcs#11 already added to Firefox profile"
                    else
                        $MODUTIL -dbdir sql:"$FIREFOX_PROFILES$i" -add "Net iD" -libfile $PKCS11LIB -mechanisms FRIENDLY -force > /dev/null 2>&1
                        $MODUTIL -dbdir "$FIREFOX_PROFILES$i" -add "Net iD" -libfile $PKCS11LIB -mechanisms FRIENDLY -force > /dev/null 2>&1
                        FF_LIST=$($MODUTIL -dbdir "$FIREFOX_PROFILES/$i" -list | grep $PKCS11LIB)
                        if [ $(echo ${#FF_LIST}) -gt 10 ]; then 
                            echo "Module \"Net iD\" pkcs#11 added to Firefox profile"
                        fi
                        FF_LIST=$($MODUTIL -dbdir sql:"$FIREFOX_PROFILES/$i" -list | grep $PKCS11LIB)
                        if [ $(echo ${#FF_LIST}) -gt 10 ]; then 
                            echo "Module \"Net iD\" pkcs#11 added to Firefox profile"
                        fi
                    fi
                done
            fi
            if [ -d "$THUNDERBIRD_PROFILES" ]; then
                TB_PROFILES=$(ls "$THUNDERBIRD_PROFILES" | grep .default)
                for i in ${TB_PROFILES[@]}; do
                    echo "Found Thunderbird profile: '$THUNDERBIRD_PROFILES$i'"
                    TB_LIST=$($MODUTIL -dbdir "$THUNDERBIRD_PROFILES$i" -list | grep $PKCS11LIB)
                    if [ $(echo ${#TB_LIST}) -gt 10 ]; then 
                        echo "Module \"Net iD\" pkcs#11 already added to Thunderbird profile"
                    else
                        $MODUTIL -dbdir sql:"$THUNDERBIRD_PROFILES$i" -add "Net iD" -libfile $PKCS11LIB -mechanisms FRIENDLY -force > /dev/null 2>&1
                        $MODUTIL -dbdir "$THUNDERBIRD_PROFILES$i" -add "Net iD" -libfile $PKCS11LIB -mechanisms FRIENDLY -force > /dev/null 2>&1
                        TB_LIST=$($MODUTIL -dbdir "$THUNDERBIRD_PROFILES/$i" -list | grep $PKCS11LIB)
                        if [ $(echo ${#TB_LIST}) -gt 10 ]; then 
                            echo "Module \"Net iD\" pkcs#11 added to Thunderbird profile"
                        fi
                        TB_LIST=$($MODUTIL -dbdir sql:"$THUNDERBIRD_PROFILES/$i" -list | grep $PKCS11LIB)
                        if [ $(echo ${#TB_LIST}) -gt 10 ]; then 
                            echo "Module \"Net iD\" pkcs#11 added to Thunderbird profile"
                        fi
                    fi
                done
            fi
            if [ -d "$CHROME_NSSDB" ]; then
                echo "Found Chrome profile: '"$CHROME_NSSDB"'"
                CB_LIST=$($MODUTIL -dbdir "$CHROME_NSSDB" -list | grep $PKCS11LIB)
                if [ $(echo ${#CB_LIST}) -gt 10 ]; then 
                    echo "Module \"Net iD\" pkcs#11 already added to Chrome profile"
                else
                    $MODUTIL -dbdir sql:"$CHROME_NSSDB" -add "Net iD" -libfile $PKCS11LIB -mechanisms FRIENDLY -force > /dev/null 2>&1
                    $MODUTIL -dbdir "$CHROME_NSSDB" -add "Net iD" -libfile $PKCS11LIB -mechanisms FRIENDLY -force > /dev/null 2>&1
                    CB_LIST=$($MODUTIL -dbdir "$CHROME_NSSDB" -list | grep $PKCS11LIB)
                    if [ $(echo ${#CB_LIST}) -gt 10 ]; then 
                        echo "Module \"Net iD\" pkcs#11 added to Chrome profile"
                    fi
                    CB_LIST=$($MODUTIL -dbdir sql:"$CHROME_NSSDB" -list | grep $PKCS11LIB)
                    if [ $(echo ${#CB_LIST}) -gt 10 ]; then 
                        echo "Module \"Net iD\" pkcs#11 added to Chrome profile"
                    fi
                fi
            fi
        else
            echo "\"modutil\" not available"
            if [ -d "$FIREFOX_ROOT" ]; then
                if [ -d "$FIREFOX_PROFILES" ]; then
                    cd "$FIREFOX_ROOT"
                    echo "Internal configure Firefox"    
                    $PKCS11BIN -pkcs11 install -profiles "$FIREFOX_PROFILES" -root "$FIREFOX_ROOT"
                fi
                if [ -d "$CHROME_NSSDB" ]; then
                    cd "$FIREFOX_ROOT"
                    echo "Internal configure Chrome"    
                    $PKCS11BIN -pkcs11 install -profiles "$CHROME_NSSDB" -root "$FIREFOX_ROOT"
                fi
            fi
        fi
    fi
fi
