#!/bin/bash

if [ $(uname) = "Darwin" ]; then 
    MODUTIL=/opt/local/bin/nss-modutil
    LIBEXT=dylib
    FIREFOX_PROFILES="$HOME/Library/Application Support/Firefox/Profiles/"
    THUNDERBIRD_PROFILES="$HOME/Library/Thunderbird/Profiles/"
    CHROME_NSSDB=$HOME/.pki/nssdb
else  
    MODUTIL=/usr/bin/modutil
    LIBEXT=so
    FIREFOX_PROFILES=$HOME/.mozilla/firefox/
    THUNDERBIRD_PROFILES="$HOME/.thunderbird/"
    CHROME_NSSDB=$HOME/.pki/nssdb
fi

PKCS11LIB=libiidp11.$LIBEXT

if [ -f $MODUTIL ]; then
    if [ -d "$FIREFOX_PROFILES" ]; then
        FF_PROFILES=$(ls "$FIREFOX_PROFILES" | grep .default)
        for i in ${FF_PROFILES[@]}; do
            echo "Found Firefox profile '$FIREFOX_PROFILES$i'"
            FF_LIST=$($MODUTIL -dbdir "$FIREFOX_PROFILES$i" -list | grep $PKCS11LIB)
            if [ $(echo ${#FF_LIST}) -gt 10 ]; then 
                $MODUTIL -dbdir "$FIREFOX_PROFILES$i" -delete "Net iD" -force
            fi
            FF_LIST=$($MODUTIL -dbdir sql:"$FIREFOX_PROFILES$i" -list | grep $PKCS11LIB)
            if [ $(echo ${#FF_LIST}) -gt 10 ]; then 
                $MODUTIL -dbdir sql:"$FIREFOX_PROFILES$i" -delete "Net iD" -force
            fi
        done
    fi
    if [ -d "$THUNDERBIRD_PROFILES" ]; then
        TB_PROFILES=$(ls "$THUNDERBIRD_PROFILES" | grep .default)
        for i in ${TB_PROFILES[@]}; do
            echo "Found Thunderbird profile '$THUNDERBIRD_PROFILES$i'"
            TB_LIST=$($MODUTIL -dbdir "$THUNDERBIRD_PROFILES$i" -list | grep $PKCS11LIB)
            if [ $(echo ${#TB_LIST}) -gt 10 ]; then 
                $MODUTIL -dbdir "$THUNDERBIRD_PROFILES$i" -delete "Net iD" -force
            fi
            TB_LIST=$($MODUTIL -dbdir sql:"$THUNDERBIRD_PROFILES$i" -list | grep $PKCS11LIB)
            if [ $(echo ${#TB_LIST}) -gt 10 ]; then 
                $MODUTIL -dbdir sql:"$THUNDERBIRD_PROFILES$i" -delete "Net iD" -force
            fi
        done
    fi
    if [ -d "$CHROME_NSSDB" ]; then
        echo "Found Chrome profile '$CHROME_NSSDB'"
        CB_LIST=$($MODUTIL -dbdir "$CHROME_NSSDB" -list | grep $PKCS11LIB)
        if [ $(echo ${#CB_LIST}) -gt 10 ]; then 
            $MODUTIL -dbdir "$CHROME_NSSDB" -delete "Net iD" -force
        fi
        CB_LIST=$($MODUTIL -dbdir sql:"$CHROME_NSSDB" -list | grep $PKCS11LIB)
        if [ $(echo ${#CB_LIST}) -gt 10 ]; then 
            $MODUTIL -dbdir sql:"$CHROME_NSSDB" -delete "Net iD" -force
        fi
    fi
fi
