//-----------------------------------------------------------------------
// Globals
//-----------------------------------------------------------------------
var SERVICES = null;
var CERTIFICATES = null;
var SLOTS = null;
var USERS = null;
var REQUESTS = null;
//-----------------------------------------------------------------------
// OnLoad
//-----------------------------------------------------------------------
function OnLoad() {
    var vars = null;
    // Capture key F5 (=refresh), will call OnReload below
    SetDetectReload();
    // May need to load more variables, so only continue if non-null
    if ((vars = GetVariables("index")) != null) {
        OnLoadVariables(vars);
    }
    return;
}
//-----------------------------------------------------------------------
// OnLoadVariables
//-----------------------------------------------------------------------
function OnLoadVariables(vars) {
    var value = null;
    var value2 = null;
    if (vars != null) {
        // Redirect active means that authenticate/sign request is available
        // and should be activated
        if (((vars["autostarttoken"] == null) || !DoAutoStartTokenRequest(vars["autostarttoken"])) &&
            ((vars["redirect"] == null) || !DoActivateRequest())) {
            // Set all variable parameters
            if ((value = vars["ProductInfo"]) != null) {
                if ((value2 = GetPartBy(GetPartBy(value, 5, ';'), 0, ',')) == "") {
                    value2 = GetPartBy(value, 0, ';');
                }
                SetElementLabel("title", value2);
                SetCustomFlags(GetPartBy(GetPartBy(value, 5, ';'), 3, ','));
                if (GetCustomFlagUseTitleImage()) {
                    SetElementVisible("logo_image");
                    SetElementHidden("logo_name");
                }
                else {
                    SetElementHidden("logo_image");
                    SetElementVisible("logo_name");
                }
            }
            // Process result if we got some (delayed). When processing events 
            // will we get only result variable (vars length = 1).
            if ((vars["result"] != null) && (vars.length > 1)) {
                StoreAllVariables(vars);
                OnResult();
            }
            else {
                // Always reset data store at initial page
                ResetDataStore();
                // Load start page
                OnStart();
            }
        }
    }
    return;
}
//-----------------------------------------------------------------------
// OnStart
//-----------------------------------------------------------------------
function OnStart() {
    if (!LoadPluginVariables(null, null, OnLoadVariables)) {
        OnSettings();
    }
    return;
}
//-----------------------------------------------------------------------
// OnReload
//-----------------------------------------------------------------------
function OnReload() {
    ShowProgress();
    // Either full refresh or only request
    if (GetCustomFlagSimpleRefresh()) {
        // Reload request information
        DoLoadRequests(RestoreAllVariables());
    }
    else {
        // Full reload
        OnStart();
    }
    return;
}
//-----------------------------------------------------------------------
// OnResult
//-----------------------------------------------------------------------
function OnResult() {
    var vars = null;
    var i = 0;
    var service = null;
    if ((vars = RestoreAllVariables()) != null) {
        DoLoadSlot(vars);
        DoLoadUser(vars);
        DoLoadService(vars);
        DoLoadRequests(vars);
        ShowElements(vars["ConfigGlobal:Administration:View"]);
        ShowHeader();
        setTimeout(OnTimeout, 999);
    }
    HideProgress();
    return;
}
//-----------------------------------------------------------------------
// OnTimeout
//-----------------------------------------------------------------------
function OnTimeout() {
    var count = 0;
    // Need to make sure action is unique to avoid webkit bug
    count = ConvertNumber(document.form1.count.value) + 1;
    document.form1.count.value = count;
    CallPluginComponent(document.form1.next.value, "event?" + count, "", "", "", "", OnLoadVariables);
    setTimeout(OnTimeout, 999);
    return;
}
//-----------------------------------------------------------------------
// OnRefresh
//-----------------------------------------------------------------------
function OnRefresh() {
    OnReload();
    return;
}
//-----------------------------------------------------------------------
// OnSettings
//-----------------------------------------------------------------------
function OnSettings() {
    LoadPage("support", null);
    return;
}
//-----------------------------------------------------------------------
// OnView
//-----------------------------------------------------------------------
function OnView(name) {
    if (name.substr(0, 4) == "slot") {
        OnViewSlot(parseInt(name.substr(4)));
    }
    else if (name.substr(0, 4) == "user") {
        OnViewUser(parseInt(name.substr(4)));
    }
    else if (name.substr(0, 7) == "request") {
        OnViewRequest(parseInt(name.substr(7)));
    }
    else if (name.substr(0, 7) == "service") {
        OnViewService(parseInt(name.substr(7)));
    }
    return;
}
//-----------------------------------------------------------------------
// OnViewSlot
//-----------------------------------------------------------------------
function OnViewSlot(index) {
    var vars = null;
    var slot = null;
    var i = 0;
    var j = 0;
    var count = 0;
    var count2 = 0;
    var tmp = 0;
    var value = "";
    if (SLOTS != null) {
        if ((vars = RestoreAllVariables()) != null) {
            // Get real index (only count unique slot description)
            for (i = 0; i < SLOTS.length; i++) {
                if (tmp != SLOTS[i].description) {
                    count++;
                    if (count == (index + 1)) {
                        break;
                    }
                }
                tmp = SLOTS[i].description;
            }
            // Continue if within limits
            if (i < SLOTS.length) {
                // Will send all information and use slotid to identify what 
                // should be presented
                slot = SLOTS[i];
                count = 0;
                for (i = 0; i < SLOTS.length; i++) {
                    if (SLOTS[i].present) {
                        if (slot.description == SLOTS[i].description) {
                            count++;
                        }
                    }
                }
                if (count == 0) {
                    ShowErrorPkcs11("CKR_TOKEN_NOT_PRESENT");
                }
                else {
                    value = "slotid=" + slot.slotid;
                    LoadPage("token", value);
                }
            }
        }
    }
    return;
}
//-----------------------------------------------------------------------
// OnViewUser
//-----------------------------------------------------------------------
function OnViewUser(index) {
    var i = 0;
    var count = 0;
    var value = null;
    // Find correct service element
    if (SERVICES != null) {
        for (i = 0; i < SERVICES.length; i++) {
            if (CreateUserList(RestoreAllVariables(), SERVICES[i]) != null) {
                if (index == count) {
                    value = "service=" + URL.encode(PackServiceInfo(SERVICES[index]));
                    break;
                }
                count++;
            }
        }
    }
    if (value != null) {
        LoadPage("user", value);
    }
    return;
}
//-----------------------------------------------------------------------
// OnViewService
//-----------------------------------------------------------------------
function OnViewService(index) {
    var i = 0;
    var count = 0;
    var value = null;
    var client = null;
    var url = null;
    var loaded = false;
    // Find service element
    if (SERVICES != null) {
        for (i = 0; i < SERVICES.length; i++) {
            if (SERVICES[i].site != null) {
                if (index == count) {
                    value = "info=" + URL.encode(PackServiceInfo(SERVICES[index]));
                    client = new AccessClient(SERVICES[i].address);
                    if ((url = client.GetSiteURL(SERVICES[i].site)) != null) {
                        LoadSite(url, value);
                        loaded = true;
                    }
                    break;
                }
                count++;
            }
        }
    }
    // Will be support if not loaded
    if (!loaded) {
        LoadPage("support", null);
    }
    return;
}
//-----------------------------------------------------------------------
// OnViewRequest
//-----------------------------------------------------------------------
function OnViewRequest(index) {
    var request = null;
    var user = null;
    var value = null;
    var i = 0;
    if ((REQUESTS != null) && (index < REQUESTS.length)) {
        request = REQUESTS[index];
        if ((user = GetRequestUser(request.personalNumber, request.address)) != null) {
            value = "user=" + URL.encode(PackUserInfo(user)) + "&";
            value += "request=" + URL.encode(PackRequestInfo(request));
            // Activate have own page
            if (request.type == "activate") {
                LoadPage("activate", value);
            }
            else {
                LoadPage("request", value);
            }
        }
    }
    return;
}
//-----------------------------------------------------------------------
// DoLoadSlot
//-----------------------------------------------------------------------
function DoLoadSlot(vars) {
    var listToken = null;
    var listSlot = null;
    var listCert = null;
    var i = 0;
    var j = 0;
    var count = 0;
    var tmp = "";
    var elem = null;
    var label = null;
    var name = null;
    var company = null;
    // Remove eventual old list
    RemoveElemList("slot");
    // Get list of all available tokens/slots
    listToken = CreateTokenList(vars);
    if ((listSlot = CreateSlotList(vars, listToken)) != null) {
        // Get a list of certificates, first certificate for each slot 
        // be used as name (may be empty)
        listCert = CreateCertificateList(vars, null);
        // Smart cards tokens will have one slot for each pin, so only
        // count unique slot description
        for (i = 0; i < listSlot.length; i++) {
            if (tmp != listSlot[i].description) {
                count++;
            }
            tmp = listSlot[i].description;
        }
        // Add each slot and set name
        if (count > 0) {
            CreateElemList(count, "slot", true);
            SetElementValue("slottitle", GetString("TEXT_TOKENS"));
            tmp = "";
            count = 0;
            for (i = 0; i < listSlot.length; i++) {
                if (tmp != listSlot[i].description) {
                    if (listSlot[i].present) {
                        // Use token label and manufacturer unless certificate found
                        label = StripParenthesisPart(listSlot[i].token.label);
                        name = label;
                        company = listSlot[i].token.manufacturer;
                        if (listCert != null) {
                            for (j = 0; j < listCert.length; j++) {
                                if (listCert[j].ca == "0") {
                                    if (listSlot[i].slotid == listCert[j].slotid) {
                                        name = listCert[j].name;
                                        company = listCert[j].company;
                                        break;
                                    }
                                }
                            }
                        }
                        // Insert with name from above
                        SetElementLabel("slotname" + count, name);
                    }
                    else {
                        SetElementValue("slotname" + count, GetString("TEXT_INSERT_TOKEN"));
                    }
                    if (listSlot[i].slotid < 100) {
                        SetElementLabel("slottext" + count, listSlot[i].description);
                    }
                    else {
                        SetElementHidden("slottext" + count);
                    }
                    if ((company = GetImageByName(company, false)) != null) {
                        SetElementImage("slotimage" + count, company);
                    }
                    else {
                        if (!listSlot[i].present) {
                            SetElementDefaultImage("slotimage" + count, "insert_token");
                        }
                        else if (listSlot[i].slotid < 100) {
                            SetElementDefaultImage("slotimage" + count, "token_card");
                        }
                        else {
                            SetElementDefaultImage("slotimage" + count, "token_soft");
                        }
                    }
                    SetElementHidden("slotinfo" + count);
                    count++;
                }
                tmp = listSlot[i].description;
            }
        }
        // Store slots/certificates for use when clicked (OnViewSlot)
        SLOTS = listSlot;
        CERTIFICATES = listCert;
    }
    return;
}
//-----------------------------------------------------------------------
// DoLoadUser
//-----------------------------------------------------------------------
function DoLoadUser(vars) {
    var listService = null;
    var listUser = null;
    var i = 0;
    var count = 0;
    // Remove eventual old list
    RemoveElemList("user");
    // Get list of all available services
    if ((listService = CreateServiceList(false)) != null) {
        for (i = 0; i < listService.length; i++) {
            // Only list those with connected users
            if ((listUser = CreateUserList(vars, listService[i])) != null) {
                count++;
            }
        }
        // Add each service with available user
        if (count > 0) {
            CreateElemList(count, "user", true);
            SetElementValue("usertitle", GetString("TEXT_USERS"));
            // Show service with available users
            for (i = 0; i < listService.length; i++) {
                if ((listUser = CreateUserList(vars, listService[i])) != null) {
                    SetElementLabel("username" + i, listService[i].name);
                    if ((image = GetImageByPath(listService[i].image, listService[i].address)) != null) {
                        SetElementImage("userimage" + i, image);
                        SetElementHidden("usertext" + i);
                        SetElementHidden("userinfo" + i);
                    }
                }
            }
        }
    }
    return;
}
//-----------------------------------------------------------------------
// DoLoadService
//-----------------------------------------------------------------------
function DoLoadService(vars) {
    var listService = null;
    var update = false;
    var i = 0;
    var image = 0;
    var count = 0;
    // Remove eventual old list
    RemoveElemList("service");
    // Eventually update service information
    if (GetCustomFlagUpdateService()) {
        update = true;
    }
    // Get list of all available services
    if ((listService = CreateServiceList(update)) != null) {
        for (i = 0; i < listService.length; i++) {
            // Only list those with view site
            if ((listService[i].site != null) && (listService[i].site.length > 0)) {
                count++;
            }
        }
    }
    // Create list of elements and add one for support
    count++;
    CreateElemList(count, "service", true);
    SetElementValue("servicetitle", GetString("TEXT_SERVICES"));
    // Fill list of service elements
    if (listService != null) {
        for (i = 0; i < listService.length; i++) {
            if ((listService[i].site != null) && (listService[i].site.length > 0)) {
                SetElementLabel("servicename" + i, listService[i].name);
                if ((image = GetImageByPath(listService[i].image, listService[i].address)) != null) {
                    SetElementImage("serviceimage" + i, image);
                    SetElementHidden("servicetext" + i);
                    SetElementHidden("serviceinfo" + i);
                }
            }
        }
        SERVICES = listService;
    }
    // Add support element
    i = count - 1;
    SetElementValue("servicename" + i, GetString("TITLE_SUPPORT"));
    SetElementDefaultImage("serviceimage" + i, "support");
    SetElementHidden("servicetext" + i);
    SetElementHidden("serviceinfo" + i);
    return;
}
//-----------------------------------------------------------------------
// DoLoadRequests
//-----------------------------------------------------------------------
function DoLoadRequests(vars) {
    var listSearch = null;
    // Remove eventual old list
    RemoveElemList("request");
    REQUESTS = null;
    // Get search list for requests (all active user)
    listSearch = CreateRequestSearchList(vars);
    // Search for all request. When list is searched will activate be
    // appended, so call even for empty list.
    DoSearchRequests(listSearch);
    return;
}
//-----------------------------------------------------------------------
// DoSearchRequests
//-----------------------------------------------------------------------
function DoSearchRequests(listSearch) {
    var i = 0;
    var search = null;
    if ((listSearch != null) && ((i = listSearch.indexOf(';')) != -1)) {
        search = listSearch.substr(0, i);
        listSearch = listSearch.substr(i + 1);
        DoSearchRequest(GetPartBy(search, 0, ','), GetPartBy(search, 1, ','), listSearch);
    }
    else {
        REQUESTS = AppendActivateList(REQUESTS, SERVICES);
        FillRequestList(REQUESTS);
    }
    return;
}
//-----------------------------------------------------------------------
// DoSearchRequest
//-----------------------------------------------------------------------
function DoSearchRequest(service, personalNumber, context) {
    var client = null;
    var value = null;
    client = new AccessClient(service);
    if (GetCustomFlagNoAsync()) {
        value = client.NextRequest(personalNumber);
        AppendRequest(value, context);
    }
    else {
        client.SetCallback(AppendRequest, context);
        client.NextRequest(personalNumber);
    }
    return;
}
//-----------------------------------------------------------------------
// AppendRequest
//-----------------------------------------------------------------------
function AppendRequest(request, context) {
    // Append to request list if available
    if (request != null) {
        if (REQUESTS == null) {
            REQUESTS = new Array();
        }
        REQUESTS.push(request);
    }
    // Continue with next search
    DoSearchRequests(context);
    return;
}
//-----------------------------------------------------------------------
// AddRequestUser
//-----------------------------------------------------------------------
function AddRequestUser(user, service) {
    if (USERS == null) {
        USERS = new Array();
    }
    USERS[user.personalNumber + "_" + service] = user;
    return;
}
//-----------------------------------------------------------------------
// GetRequestUser
//-----------------------------------------------------------------------
function GetRequestUser(personalNumber, service) {
    var user = null;
    if (USERS != null) {
        user = USERS[personalNumber + "_" + service];
    }
    return user;
}
//-----------------------------------------------------------------------
// CreateRequestSearchList
//-----------------------------------------------------------------------
function CreateRequestSearchList(vars) {
    var listUser = null;
    var listSearch = null;
    var i = 0;
    var j = 0;
    // Need some services to search
    if (SERVICES != null) {
        for (i = 0; i < SERVICES.length; i++) {
            // Get list of all available users
            if ((listUser = CreateUserList(vars, SERVICES[i])) != null) {
                // Check request for available users, require we have shared secret
                for (j = 0; j < listUser.length; j++) {
                    if (HasActiveUserInfo(SERVICES[i].address, listUser[j].personalNumber)) {
                        if (listSearch == null) {
                            listSearch = "";
                        }
                        listSearch += SERVICES[i].address + "," + listUser[j].personalNumber + ";";
                        AddRequestUser(listUser[j], SERVICES[i].address);
                    }
                }
            }
        }
    }
    return listSearch;
}
//-----------------------------------------------------------------------
// AppendActivateList
//-----------------------------------------------------------------------
function AppendActivateList(listRequest, listService) {
    var listUser = null;
    var i = 0;
    var j = 0;
    var service = null;
    var user = null;
    var data = null;
    var request = null;
    // Service list must be created if something is available
    if (listService != null) {
        // Enumerate all services to add eventual active request
        for (i = 0; i < listService.length; i++) {
            service = listService[i];
            if ((listUser = CreateUserList(RestoreAllVariables(), service)) != null) {
                // Check for any user need to be activated
                for (j = 0; j < listUser.length; j++) {
                    user = listUser[j];
                    if (!HasActiveRequest(listRequest, service.address, user.personalNumber)) {
                        if (!HasActiveUserInfo(service.address, user.personalNumber)) {
                            data = "<activate>";
                            data += "<server>" + service.name + "</server>";
                            data += "<user>" + user.personalNumber + "</user>";
                            data += "<date>" + FormatDate(new Date()) + "</date>";
                            data += "</activate>";
                            data = BASE64.encode(data);
                            if ((request = new RequestInfo(
                                service.address,
                                user.personalNumber,
                                user.personalNumber,
                                "activate",
                                service.name,
                                service.image,
                                null,
                                data,
                                FormatDate(new Date()))) != null) {
                                // Insert activate request to list
                                if (listRequest == null) {
                                    listRequest = new Array();
                                }
                                listRequest.push(request);
                                // Add user information
                                AddRequestUser(user, service.address);
                            }
                        }
                    }
                }
            }
        }
    }
    return listRequest;
}
//-----------------------------------------------------------------------
// FillRequestList
//-----------------------------------------------------------------------
function FillRequestList(listRequest) {
    var i = 0;
    var user = null;
    // Remove eventual old list
    RemoveElemList("request");
    // All lists created if something is available
    if ((listRequest != null) && (listRequest.length > 0)) {
        // Create elements for request
        CreateElemList(listRequest.length, "request", true);
        SetElementValue("requesttitle", GetString("TEXT_REQUESTS"));
        // Insert each element
        for (i = 0; i < listRequest.length; i++) {
            if ((user = GetRequestUser(listRequest[i].personalNumber, listRequest[i].address)) != null) {
                SetElementImage("requestimage" + i, GetImageByPath(listRequest[i].serverImage, listRequest[i].address));
                SetElementLabel("requestname" + i, user.name);
                SetElementLabel("requesttext" + i, listRequest[i].serverName);
                SetElementValue("requestinfo" + i, GetRequestString(listRequest[i].type));
            }
        }
    }
    return;
}
//-----------------------------------------------------------------------
// DoActivateRequest
//-----------------------------------------------------------------------
function DoActivateRequest() {
    var success = false;
    var listService = null;
    var listUser = null;
    var client = null;
    var store = null;
    var i = 0;
    var j = 0;
    var request = null;
    var count = 0;
    // No need to ask plugin for information, since we should have everything
    // we need in cache. Search all active user after first available request
    // and active it. For failure go to start page.
    if ((listService = CreateServiceList(false)) != null) {
        store = new DataStore();
        for (i = 0; i < listService.length; i++) {
            client = new AccessClient(listService[i].address);
            if ((listUser = CreateActiveUserList(listService[i].address)) != null) {
                for (j = 0; j < listUser.length; j++) {
                    if ((request = client.NextRequest(listUser[j].personalNumber)) != null) {
                        // Store request information so request page can have it
                        count++;
                        store.setValue("request" + count, PackRequestInfo(request));
                    }
                }
                if (count > 0) {
                    // Need to get all available certificates (and token info)
                    document.form1.next.value = "request.html";
                    LoadPluginVariables(null, null, null);
                    success = true;
                }
            }
        }
    }
    return success;
}
//-----------------------------------------------------------------------
// DoAutoStartTokenRequest
//-----------------------------------------------------------------------
function DoAutoStartTokenRequest(orderRef) {
    var success = false;
    var listService = null;
    var listUser = null;
    var client = null;
    var store = null;
    var i = 0;
    var j = 0;
    var request = null;
    var count = 0;
    // No need to ask plugin for information, since we should have everything
    // we need in cache. Take first available user, and search all active 
    // services for first service that accept the specified orderRef. For
    // failure go to start page.
    if ((listService = CreateServiceList(false)) != null) {
        store = new DataStore();
        for (i = 0; i < listService.length; i++) {
            client = new AccessClient(listService[i].address);
            if ((listUser = CreateActiveUserList(listService[i].address)) != null) {
                for (j = 0; j < listUser.length; j++) {
                    if ((request = client.RegisterRequest(orderRef, listUser[j].personalNumber)) != null) {
                        // Store request information so request page can have it
                        count++;
                        store.setValue("request" + count, PackRequestInfo(request));
                        break;
                    }
                }
                if (count > 0) {
                    // Need to get all available certificates (and token info)
                    document.form1.next.value = "request.html";
                    LoadPluginVariables(null, null, null);
                    success = true;
                }
            }
        }
    }
    return success;
}
//-----------------------------------------------------------------------
// ShowElements
//-----------------------------------------------------------------------
function ShowElements(view) {
    view = GetPartBy(GetPartBy(view, 0, ';'), 0, ',');
    ShowElementList(view, "requests;slots;services;users;btn_settings;btn_refresh");
    return;
}
