//-----------------------------------------------------------------------
// Globals
//-----------------------------------------------------------------------
var USER = null;
var REQUEST = null;
var RESULT = null;
var SIGNATURE = null;
//-----------------------------------------------------------------------
// OnLoad
//-----------------------------------------------------------------------
function OnLoad() {
    var vars = null;
    // Capture key F5 (=refresh), will call OnReload below
    SetDetectReload();
    // Get our variables
    if ((vars = GetVariables("request")) != null) {
        SetCurrentLanguage(vars);
        OnLoadVariables(vars);
    }
    return;
}
//-----------------------------------------------------------------------
// OnLoadVariables
//-----------------------------------------------------------------------
function OnLoadVariables(vars) {
    var store = null;
    var result = null;
    var signature = null;
    var requestInfo = null;
    var request = null;
    var userInfo = null;
    var user = null;
    var certificate = null;
    var service = null;
    var count = 0;
    // Require variables
    if (vars != null) {
        store = new DataStore();
        // Only get result value when event is available
        if (vars.length == 1) {
            OnReload();
        }
        // For direct activation will we get a list of certificates instead of
        // the user
        else if (vars["CertificateExCount"] != null) {
            store.setValue("user", null);
            store.setValue("request", null);
            while (true) {
                count++;
                if ((requestInfo = store.getValue("request" + count)) == null) {
                    break;
                }
                if ((request = ParseRequestInfo(requestInfo)) != null) {
                    store.setValue("request", requestInfo);
                    if ((service = GetServiceInfo(request.address)) != null) {
                        if ((userInfo = LoadUserInfo(service, request, vars)) != null) {
                            store.setValue("user", userInfo);
                            break;
                        }
                    }
                }
            }
        }
        else {
            // Or we should get request and user at first call, store for later use
            if ((userInfo = vars["user"]) != null) {
                store.setValue("user", userInfo);
            }
            if ((requestInfo = vars["request"]) != null) {
                store.setValue("request", requestInfo);
            }
            // When creating signatures will we get result
            result = vars["result"];
        }
        // We should have a request and a user, this site would only be called
        // with active request, so failure means user is missing, i.e. smart 
        // card not present. 
        userInfo = store.getValue("user");
        requestInfo = store.getValue("request");
        if (userInfo != null) {
            user = ParseUserInfo(userInfo);
        }
        if (requestInfo != null) {
            request = ParseRequestInfo(requestInfo);
        }
        if (request == null) {
            ShowError(GetString("ERROR_SERVICE_REQUEST_MISSING"), null);
        }
        else if (user == null) {
            SetInformationInsert(request);
            setTimeout(OnInsertTimer, 999);
            REQUEST = request;
        }
        else {
            USER = user;
            REQUEST = request;
            signature = URL.decode(vars["Signature"]);
            SetInformationRequest(user, request);
        }
        // Handle result when available
        if (result != null) {
            RESULT = result;
            SIGNATURE = signature;
            ShowProgress();
            setTimeout(OnDelayedResult, 111);
        }
        // Show whole content
        ShowContent();
        // Set focus on password field at start
        if (result == null) {
            SetFieldFocus("pwd");
        }
    }
    return;
}
//-----------------------------------------------------------------------
// OnInsertTimer
//-----------------------------------------------------------------------
function OnInsertTimer() {
    var count = 0;
    // Need to make sure action is unique to avoid webkit bug
    count = ConvertNumber(document.form1.count.value) + 1;
    document.form1.count.value = count;
    CallPluginComponent(document.form1.next.value, "event?" + count, "", "", "", "", OnLoadVariables);
    setTimeout(OnInsertTimer, 999);
    return;
}
//-----------------------------------------------------------------------
// OnReload
//-----------------------------------------------------------------------
function OnReload() {
    LoadPluginVariables(null, null, OnLoadVariables);
    return;
}
//-----------------------------------------------------------------------
// ShowContent
//-----------------------------------------------------------------------
function ShowContent() {
    ShowHeader();
    SetElementVisible("content");
    return;
}
//-----------------------------------------------------------------------
// LoadUserInfo
//-----------------------------------------------------------------------
function LoadUserInfo(service, request, vars) {
    var userInfo = null;
    var listUser = null;
    var i = 0;
    // Get list of all available users
    if ((listUser = CreateUserList(vars, service)) != null) {
        for (i = 0; i < listUser.length; i++) {
            if (listUser[i].personalNumber == request.personalNumber) {
                userInfo = PackUserInfo(listUser[i]);
            }
        }
    }
    return userInfo;
}
//-----------------------------------------------------------------------
// OnDelayedResult
//-----------------------------------------------------------------------
function OnDelayedResult() {
    OnResult(RESULT, SIGNATURE);
    return;
}
//-----------------------------------------------------------------------
// OnResult
//-----------------------------------------------------------------------
function OnResult(result, signature) {
    if ((result == "CKR_OK") && (signature != null)) {
        OnProcess(REQUEST, "done", signature);
    }
    else if (result == "CKR_CANCEL") {
        OnProcess(REQUEST, "cancel", null);
    }
    else if (result == "CKR_PIN_LOCKED") {
        OnProcess(REQUEST, "locked", null);
    }
    else {
        HandleResult(result);
    }
    return;
}
//-----------------------------------------------------------------------
// OnProcess
//-----------------------------------------------------------------------
function OnProcess(request, action, signature) {
    var store = new DataStore();
    var client = null;
    var info = null;
    SetElementHidden("label_password");
    SetElementHidden("pwd");
    ShowProgress();
    if (request != null) {
        info = new DeviceInfo(null);
        client = new AccessClient(request.address);
        client.callback = OnProcessDone;
        client.ProcessRequest(request.orderRef, action, signature, info.device);
        USER = null;
        REQUEST = null;
        store.setValue("user", null);
        store.setValue("request", null);
    }
    return;
}
//-----------------------------------------------------------------------
// OnProcessDone
//-----------------------------------------------------------------------
function OnProcessDone(status) {
    setTimeout(OnProcessDoneDelayed, 999); 
    return;
}
//-----------------------------------------------------------------------
// OnProcessDoneDelayed
//-----------------------------------------------------------------------
function OnProcessDoneDelayed() {
    CallPluginComponent("index", "reload", "", "", "", "", null);
    return;
}
//-----------------------------------------------------------------------
// SetInformationRequest
//-----------------------------------------------------------------------
function SetInformationRequest(user, request) {
    SetElementValue("title", GetRequestString(request.type));
    SetElementValue("text_btn_ok", GetRequestString(request.type));
    SetElementLabel("companyName", request.serverName);
    SetElementImage("companyImage", GetImageByPath(request.serverImage, request.address));
    SetElementLabel("userName", user.name);
    SetElementImage("userImage", GetBigImageByName(user.company, true));
    SetElementValue("personalNumber", FormatPersonalNumber(user.personalNumber));
    if (request.type == "signature") {
        SetElementLabel("tokenName", GetPartBy(user.token, 1, ","));
    }
    else {
        SetElementLabel("tokenName", GetPartBy(user.token, 0, ","));
    }
    SetElementLabel("label_password", GetString("TEXT_PIN"));
    return;
}
//-----------------------------------------------------------------------
// SetInformationInsert
//-----------------------------------------------------------------------
function SetInformationInsert(request) {
    var user = null;
    SetElementValue("title", GetRequestString(request.type));
    SetElementValue("text_btn_ok", GetRequestString(request.type));
    SetElementLabel("companyName", request.serverName);
    SetElementImage("companyImage", GetImageByPath(request.serverImage, request.address));
    if ((user = GetActiveUserInfo(request.address, request.personalNumber)) != null) {
        SetElementLabel("userName", user.name);
        SetElementImage("userImage", GetBigImageByName(user.company, true));
        SetElementValue("personalNumber", FormatPersonalNumber(user.personalNumber));
        SetElementHidden("tokenName");
        SetElementHidden("pwd");
        SetElementHidden("progress_on");
        SetElementHidden("progress_off");
        SetElementValue("insertName", GetString("TEXT_INSERT_TOKEN"));
        SetElementVisible("insert");
    }
    return;
}
//-----------------------------------------------------------------------
// OnCancel
//-----------------------------------------------------------------------
function OnCancel() {
    OnResult("CKR_CANCEL", null);
    return;
}
//-----------------------------------------------------------------------
// OnBack
//-----------------------------------------------------------------------
function OnBack() {
    LoadPage("index", null);
    return;
}
//-----------------------------------------------------------------------
// OnButtonView
//-----------------------------------------------------------------------
function OnButtonView() {
    var store = new DataStore();
    var data = null;
    var value = "";
    if ((REQUEST.visibleData != null) && (REQUEST.visibleData.length > 0)) {
        data = REQUEST.visibleData;
    }
    else {
        data = REQUEST.nonVisibleData;
    }
    value += "back=request&";
    value += "title=" + URL.encode(GetRequestString(REQUEST.type)) + "&";
    value += "data=" + URL.encode(data);
    LoadPage("view", value);
    return;
}
//-----------------------------------------------------------------------
// OnButtonOK
//-----------------------------------------------------------------------
function OnButtonOK() {
    SetFieldUnfocus("pwd");
    ShowProgress();
    setTimeout(OnSign, 111);
    return;
}
//-----------------------------------------------------------------------
// OnSign
//-----------------------------------------------------------------------
function OnSign() {
    var address = null;
    var type = null;
    var data = null;
    var certificate = null;
    address = REQUEST.address;
    type = REQUEST.type;
    if ((REQUEST.visibleData != null) && (REQUEST.visibleData.length > 0)) {
        data = REQUEST.visibleData;
    }
    else {
        data = REQUEST.nonVisibleData;
    }
    if (type == "signature") {
        certificate = GetPartBy(GetPartBy(USER.certificateID, 1, ','), 0, '|')
    }
    else {
        certificate = GetPartBy(GetPartBy(USER.certificateID, 0, ','), 0, '|')
    }
    DoActionSign(address, type, certificate, data);
    return;
}
//-----------------------------------------------------------------------
// HandleResult
//-----------------------------------------------------------------------
function HandleResult(result) {
    if (result != "CKR_OK") {
        ShowErrorPkcs11(result);
    }
    HideProgress();
    return;
}
//-----------------------------------------------------------------------
// DoActionSign
//-----------------------------------------------------------------------
function DoActionSign(address, type, certificate, data) {
    var value = "";
    var client = null;
    // Need a pin value
    if (document.form1.pwd.value.length == 0) {
        HandleResult("CKR_PIN_INVALID");
    }
    else {
        // Create an access client object so we can get server URL
        client = new AccessClient(address);
        // Set plugin parameters
        value += "Server=" + URL.encode(client.GetURL()) + ";";
        value += "Base64=true;";
        value += "UrlEncode=true;";
        value += "Certificate=" + URL.encode(certificate) + ";";
        value += "Password=" + URL.encode(document.form1.pwd.value) + ";";
        value += "Data=" + data + ";";
        if (type == "signature") {
            value += "Authentication=false;";
            value += "IncludeHostInfo=false;";
        }
        else {
            value += "Authentication=true;";
            value += "IncludeHostInfo=true;";
        }
        CallPluginComponent(document.form1.next.value, "plugin", value, "Sign", "", "Signature", OnLoadVariables);     
    }
    return;
}
