//-----------------------------------------------------------------------
// Globals
//-----------------------------------------------------------------------
var SELECT_ACTIVE = false;
var TRACE_ACTIVE = false;
var IGNORE_SELECT_ABORT = false;
//-----------------------------------------------------------------------
// OnLoad
//-----------------------------------------------------------------------
function OnLoad() {
    var vars = null;
    var info = null;
    if ((info = new DeviceInfo()) != null) {
        SetDeviceInfo(info);
    }
    if ((vars = GetVariables("support")) != null) {
        OnLoadVariables(vars);
    }
    return;
}
//-----------------------------------------------------------------------
// OnLoadVariables
//-----------------------------------------------------------------------
function OnLoadVariables(vars) {
    if (vars != null) {
        if (vars.length == 0) {
            vars = RestoreAllVariables();
        }
        else {
            vars = AppendToAllVariables(vars);
        }
        SetElementValue("title", GetString("TITLE_SUPPORT"));
        SetElementValue("contact_name", GetString("TEXT_CONTACT"));
        SetElementValue("settings_name", GetString("TEXT_SETTINGS"));
        SetElementValue("language_name", GetString("TEXT_LANGUAGE"));
        SetElementValue("trace_name", GetString("TEXT_TRACE"));
        SetElementValue("trace_text", GetString("INFO_TRACE_ENABLE"));
        SetElementValue("service_name", GetString("TEXT_SERVICES"));
        SetElementValue("service_text", GetString("INFO_SERVICES"));
        SetElementValue("advanced_name", GetString("TEXT_SETTINGS_ADVANCED"));
        SetElementValue("advanced_text", GetString("INFO_SETTINGS_ADVANCED"));
        SetElementDefaultImage("advanced_image", "advanced");
        ShowElements(vars["ConfigGlobal:Administration:View"]);
        SetContactInfo(vars["SupportInfo"]);
        SetTraceInfo(vars["SupportInfo"]);
        FillSelectList(vars);
        ShowHeader();
        SetElementVisible("content");
    }
    HideProgress();
    return;
}
//-----------------------------------------------------------------------
// SetDeviceInfo
//-----------------------------------------------------------------------
function SetDeviceInfo(info) {
    var value = null;
    if (info != null) {
        SetElementLabel("name", GetDevicePresentation(info.device));
        SetElementLabel("text", GetBrowserPresentation(info.browser) + ", " + GetVersionGUI());
        SetElementDefaultImage("image", "computer");
    }
    return;
}
//-----------------------------------------------------------------------
// SetContactInfo
//-----------------------------------------------------------------------
function SetContactInfo(info) {
    var available = false;
    var value = null;
    var i = 0;
    var count = 0;
    if ((info != null) && (info.length > 0)) {
        for (i = 0; i < 3; i++) {
            count++;
            if ((value = GetPartBy(info, i, ';')) != "") {
                value = URL.decode(value);
                if (i == 0) {
                    value = FormatLinkWeb(value);
                }
                else if (i == 1) {
                    value = FormatLinkMail(value);
                }
                else {
                    value = FormatLinkCopy(value);
                }
                SetElementValue("contact_text" + count, value);
                available = true;
            }
            else {
                SetElementValue("contact_text" + count, "&nbsp;");
            }
        }
    }
    if (!available) {
        SetElementHidden("contact");
    }
    else {
        SetElementDefaultImage("contact_image", "contact");
    }
    return;
}
//-----------------------------------------------------------------------
// SetTraceInfo
//-----------------------------------------------------------------------
function SetTraceInfo(info) {
    var enabled = false;
    var value = null;
    var i = 0;
    if ((info != null) && (info.length > 0)) {
        if ((value = GetPartBy(info, 3, ';')) != "") {
            if (value.toLowerCase() == "true") {
                enabled = true;
            }
        }
    }
    SetElementDefaultImage("trace_image", "trace");
    SetTraceElements(enabled);
    TRACE_ACTIVE = enabled;
    return;
}
//-----------------------------------------------------------------------
// SetTraceElements
//-----------------------------------------------------------------------
function SetTraceElements(active) {
    if (active) {
        SetElementClass("trace_toogle", "tile_icon_checkbox_01");
        SetElementValue("trace_text", GetString("INFO_TRACE_DISABLE"));
        SetElementValue("trace_open_text", GetString("BUTTON_OPEN"));
        SetElementVisible("trace_open");
        SetElementHidden("trace_text");
    }
    else {
        SetElementClass("trace_toogle", "tile_icon_checkbox_00");
        SetElementValue("trace_text", GetString("INFO_TRACE_ENABLE"));
        SetElementHidden("trace_open");
        SetElementVisible("trace_text");
    }
    return;
}
//-----------------------------------------------------------------------
// GetDevicePresentation
//-----------------------------------------------------------------------
function GetDevicePresentation(info) {
    var value = null;
    if (info.name == "Windows NT") {
        if (info.version == "5.0") {
            value = "Windows 2000";
        }
        else if (info.version == "5.1") {
            value = "Windows XP";
        }
        else if (info.version == "5.2") {
            value = "Windows 2003";
        }
        else if (info.version == "6.0") {
            value = "Windows Vista";
        }
        else if (info.version == "6.1") {
            value = "Windows 7";
        }
        else if (info.version == "6.2") {
            value = "Windows 8";
        }
        else if (info.version == "6.3") {
            value = "Windows 8.1";
        }
    }
    else if (info.name == "Macintosh") {
        value = "Mac OS X " + info.version;
    }
    else if (info.name == "iPhone OS") {
        value = "iPhone " + info.version;
    }
    else if (info.name == "iPad OS") {
        value = "iPad " + info.version;
    }
    else if (info.name == "Linux") {
        // Version info not available
        value = "Linux";
    }
    if (value == null) {
        value = info.name + " " + info.version;
    }
    return value;
}
//-----------------------------------------------------------------------
// GetDeviceImage
//-----------------------------------------------------------------------
function GetDeviceImage(info) {
    var value = null;
    if (info.name.indexOf("Windows") != -1) {
        value = "win";
    }
    else if (info.name.indexOf("Linux") != -1) {
        value = "linux";
    }
    else if ((info.name.indexOf("Macintosh") != -1) ||
             (info.name.indexOf("iPhone") != -1) ||
             (info.name.indexOf("iPad") != -1)) {
        value = "macos";
    }
    if (value != null) {
        value = "tile_os_" + value + ".png";
    }
    return value;
}
//-----------------------------------------------------------------------
// GetBrowserPresentation
//-----------------------------------------------------------------------
function GetBrowserPresentation(info) {
    return info.name + " " + info.version;
}
//-----------------------------------------------------------------------
// EnableButton
//-----------------------------------------------------------------------
function EnableButton(vars, device) {
    var value = null;
    if (((value = vars["SupportInfo"]) == null) || (value.length == 0)) {
        SetElementHidden("contact");
    }
    if ((device.name != "Windows NT") && (device.name != "Macintosh")) {
        SetElementHidden("uninstall");
    }
    if (((value = vars["TraceFile"]) == null) || (value.length == 0)) {
    }
    return;
}
//-----------------------------------------------------------------------
// OnBack
//-----------------------------------------------------------------------
function OnBack() {
    LoadPage("index", null);
    return;
}
//-----------------------------------------------------------------------
// OnRefresh
//-----------------------------------------------------------------------
function OnRefresh() {
    return;
}
//-----------------------------------------------------------------------
// OnAbout
//-----------------------------------------------------------------------
function OnAbout() {
    LoadPage("about", null);
    return;
}
//-----------------------------------------------------------------------
// OnCopyAll
//-----------------------------------------------------------------------
function OnCopyAll() {
    CopyClipboard(navigator.userAgent);
    return;
}
//-----------------------------------------------------------------------
// OnCopy
//-----------------------------------------------------------------------
function OnCopy(text) {
    CopyClipboard(text);
    return;
}
//-----------------------------------------------------------------------
// OnService
//-----------------------------------------------------------------------
function OnService() {
    LoadPage("support_adv_service", "back=support");
    return;
}
//-----------------------------------------------------------------------
// OnTraceToggle
//-----------------------------------------------------------------------
function OnTraceToggle() {
    ShowProgress();
    document.form1.next.value = "support.html";
    if (TRACE_ACTIVE) {
        TRACE_ACTIVE = false;
        LoadPluginVariables(null, "DisableTrace", OnLoadVariables);
    }
    else {
        TRACE_ACTIVE = true;
        LoadPluginVariables(null, "EnableTrace", OnLoadVariables);
    }
    SetTraceElements(TRACE_ACTIVE);
    return;
}
//-----------------------------------------------------------------------
// OnTraceOpen
//-----------------------------------------------------------------------
function OnTraceOpen() {
    var info = null;
    var win = null;
    document.form1.next.value = "none";
    if ((info = new DeviceInfo(null)) != null) {
        if (info.device.name.indexOf("Linux") != -1) {
            win = window.open("file:///tmp/iid.txt", "_blank");
            win.focus();
        }
        else {
            CallPluginComponent(document.form1.next.value, "plugin", "", "OpenTrace", "", "", OnLoadVariables)
        }
    }
    return;
}
//-----------------------------------------------------------------------
// OnSelectClick
//-----------------------------------------------------------------------
function OnSelectClick() {
    IGNORE_SELECT_ABORT = true;
    if (SELECT_ACTIVE) {
        SELECT_ACTIVE = false;
        SetElementClass("select", "select");
    }
    else {
        SELECT_ACTIVE = true;
        SetElementClass("select", "select_active");
    }
    return;
}
//-----------------------------------------------------------------------
// OnSelectAbort
//-----------------------------------------------------------------------
function OnSelectAbort() {
    if (!IGNORE_SELECT_ABORT) {
        SELECT_ACTIVE = false;
        SetElementClass("select", "select");
    }
    IGNORE_SELECT_ABORT = false;
    return;
}
//-----------------------------------------------------------------------
// OnSelectChange
//-----------------------------------------------------------------------
function OnSelectChange(value) {
    SetElementValue("language_current", value);
    SELECT_ACTIVE = false;
    SetElementClass("select", "select");
    if (value != GetCurrentLanguage()) {
        ShowProgress();
        ReloadStringTable(value);
        // Reload plugin variables and set new language for product
        document.form1.next.value = "support.html";
        value = "Config:Language:Current=" + URL.encode(value);
        LoadPluginVariables(value, null, OnLoadVariables);
    }
    return;
}
//-----------------------------------------------------------------------
// FillSelectList
//-----------------------------------------------------------------------
function FillSelectList(vars) {
    var languages = null;
    var i = 0;
    var count = 0;
    var value = null;
    var elem = null;
    var elemValue = null;
    var elemInfo = "";
    // Need to get base element value
    if ((elem = document.getElementById("elem0")) != null) {
        if ((languages = CreateLanguageList(vars)) != null) {
            // Fill list of available languages (require present both internally and in product)
            while ((value = GetPartBy(languages, i, ',')) != "") {
                value = URL.decode(value);
                if (IsLanguageAvailable(value)) {
                    count++;
                    // Create element list element
                    elemValue = elem.innerHTML;
                    elemValue = ReplaceAll(elemValue, "ELEM_VALUE", value);
                    elemValue = ReplaceAll(elemValue, "ELEM_TEXT", FormatHTML(value));
                    // Append element to list of elements
                    if (elemInfo.indexOf(elemValue) == -1) {
                        elemInfo += elemValue;
                    }
                }
                i++;
            }
        }
        // Set new element value
        elem.innerHTML = elemInfo;
        // Set current language
        SetElementLabel("language_current", GetCurrentLanguage());
        SetElementDefaultImage("language_image", "language");
    }
    if (count == 0) {
        SetElementHidden("language");
    }
    return;
}
//-----------------------------------------------------------------------
// OnAdvanced
//-----------------------------------------------------------------------
function OnAdvanced() {
    var value = null;
    value = PackAllVariables(RestoreAllVariables());
    value += "&back=support";
    LoadPage("support_adv", value);
    return;
}
//-----------------------------------------------------------------------
// ShowElements
//-----------------------------------------------------------------------
function ShowElements(view) {
    view = GetPartBy(GetPartBy(view, 0, ';'), 3, ',');
    ShowElementList(view, "contact;settings;settings_language;settings_trace;settings_service;settings_advanced");
    return;
}
